@@ -1,15 +1,27 @@
-from airflow_kubernetes_job_operator.kubernetes_job_operator import KubernetesJobOperator
-from typing import List
-import os
-from airflow import configuration
-from airflow.contrib.kubernetes.kubernetes_request_factory import pod_request_factory
-from airflow.contrib.kubernetes import pod_generator
-from airflow.contrib.kubernetes.pod import Resources
-from airflow.contrib.kubernetes.volume_mount import VolumeMount
-from airflow.contrib.kubernetes.volume import Volume
-from airflow.contrib.kubernetes.secret import Secret
+import kubernetes.client as k8s
 
-JOB_YAML_DEFAULT_FILE = os.path.abspath(f"{__file__}.job.yaml")
+from typing import List, Optional, Union
+from airflow import configuration
+from airflow_kubernetes_job_operator.job_runner import JobRunnerDeletePolicy
+from airflow_kubernetes_job_operator.kubernetes_job_operator import KubernetesJobOperator
+
+try:
+    from airflow.contrib.kubernetes.kubernetes_request_factory import pod_request_factory
+    from airflow.contrib.kubernetes import pod_generator
+    from airflow.contrib.kubernetes.pod import Resources
+    from airflow.contrib.kubernetes.volume_mount import VolumeMount
+    from airflow.contrib.kubernetes.volume import Volume
+    from airflow.contrib.kubernetes.secret import Secret
+except Exception:
+    from kubernetes.client import (
+        V1ResourceRequirements as Resources,
+        V1Volume as Volume,
+        V1VolumeMount as VolumeMount,
+        V1Secret as Secret,
+    )
+
+    pod_generator = None
+    pod_request_factory = None
 
 
 class KubernetesLegacyJobOperator(KubernetesJobOperator):
@@ -17,7 +29,6 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
         self,
         namespace: str = None,
         image: str = None,
-        name: str = None,
         cmds: List[str] = None,
         arguments: List[str] = None,
         ports: list = None,
@@ -46,10 +57,16 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
         pod_runtime_info_envs: dict = None,
         dnspolicy: str = None,
         ## new args.
-        job_yaml: str = None,
-        job_yaml_filepath: str = None,
-        delete_policy: str = None,
-        validate_yaml_on_init: bool = None,
+        init_containers: Optional[List[k8s.V1Container]] = None,
+        env_from: List[str] = None,
+        schedulername: str = None,
+        priority_class_name: str = None,
+        # job operator args
+        body: str = None,
+        body_filepath: str = None,
+        delete_policy: Union[str, JobRunnerDeletePolicy] = None,
+        validate_body_on_init: bool = None,
+        wait_for_task_timeout: float = None,
         *args,
         **kwargs,
     ):
@@ -136,21 +153,23 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
 
         Added arguments:
 
-            job_yaml {dict|string} -- The job to execute as a yaml description. (default: None)
-            job_yaml_filepath {str} -- The path to the file to read the yaml from, overridden by
-                job_yaml. (default: None)
+            body {dict|string} -- The job to execute as a yaml description. (default: None)
+            body_filepath {str} -- The path to the file to read the yaml from, overridden by
+                body. (default: None)
             delete_policy {str} -- Any of: Never, Always, IfSucceeded (default: {"IfSucceeded"});
                 overrides is_delete_operator_pod.
-            validate_yaml_on_init {bool} -- If true, validates the yaml in the constructor,
+            validate_body_on_init {bool} -- If true, validates the yaml in the constructor,
                 setting this to True, will slow dag creation.
-                (default: {from env/airflow config: AIRFLOW__KUBE_JOB_OPERATOR__VALIDATE_YAML_ON_INIT or False})
+                (default: {from env/airflow config: AIRFLOW__KUBE_JOB_OPERATOR__validate_body_on_init or False})
         """
-        delete_policy = delete_policy or "IfSucceeded" if is_delete_operator_pod else "Never"
-        validate_yaml_on_init = (
-            configuration.conf.getboolean(
-                "kube_job_operator", "VALIDATE_YAML_ON_INIT", fallback=False
-            )
-            or False
+        delete_policy = (
+            delete_policy or JobRunnerDeletePolicy.IfSucceeded
+            if is_delete_operator_pod
+            else JobRunnerDeletePolicy.Never
+        )
+
+        validate_body_on_init = (
+            configuration.conf.getboolean("kube_job_operator", "validate_body_on_init", fallback=False) or False
         )
 
         super().__init__(
@@ -158,15 +177,15 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
             arguments=arguments,
             image=image,
             namespace=namespace,
-            name=name,
-            job_yaml=job_yaml,
-            job_yaml_filepath=job_yaml_filepath,
+            body=body,
+            body_filepath=body_filepath,
             delete_policy=delete_policy,
             in_cluster=in_cluster,
             config_file=config_file,
             cluster_context=cluster_context,
-            validate_yaml_on_init=validate_yaml_on_init,
+            validate_body_on_init=validate_body_on_init,
             startup_timeout_seconds=startup_timeout_seconds,
+            wait_for_task_timeout=wait_for_task_timeout,
             get_logs=get_logs,
             *args,
             **kwargs,
@@ -179,7 +198,6 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
         self.arguments = arguments or []
         self.labels = labels or {}
         self.startup_timeout_seconds = startup_timeout_seconds
-        self.name = name
         self.env_vars = env_vars or {}
         self.ports = ports or []
         self.volume_mounts = volume_mounts or []
@@ -199,6 +217,11 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
         self.pod_runtime_info_envs = pod_runtime_info_envs or []
         self.dnspolicy = dnspolicy
 
+        self.init_containers = init_containers
+        self.env_from = env_from
+        self.schedulername = schedulername
+        self.priority_class_name = priority_class_name
+
     def _set_resources(self, resources):
         # Legacy
         inputResource = Resources()
@@ -207,56 +230,105 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
                 setattr(inputResource, item, resources[item])
         return inputResource
 
-    def pre_execute(self, context):
-        # creating the job yaml.
-        super().pre_execute(context)
+    def prepare_and_update_body(self):
+        self.job_runner.prepare_body()
 
-        gen = pod_generator.PodGenerator()
+        pod_body = None
+        if pod_generator is not None:
+            # old pod generator
+            gen = pod_generator.PodGenerator()
 
-        for port in self.ports:
-            gen.add_port(port)
-        for mount in self.volume_mounts:
-            gen.add_mount(mount)
-        for volume in self.volumes:
-            gen.add_volume(volume)
+            for port in self.ports:
+                gen.add_port(port)
+            for mount in self.volume_mounts:
+                gen.add_mount(mount)
+            for volume in self.volumes:
+                gen.add_volume(volume)
 
-        # selecting appropriate pod values.
-        all_labels = {}
-        all_labels.update(self.labels)
-        all_labels.update(self.job_yaml["spec"]["template"]["metadata"]["labels"])
-        image = self.image or self.job_yaml["spec"]["template"]["spec"]["containers"][0]["image"]
-        cmds = self.cmds or self.job_yaml["spec"]["template"]["spec"]["containers"][0]["command"]
-        arguments = (
-            self.arguments or self.job_yaml["spec"]["template"]["spec"]["containers"][0]["args"]
-        )
+            job_obj = self.body[0]
 
-        pod = gen.make_pod(
-            namespace=self.job_yaml["metadata"]["namespace"],
-            image=image,
-            pod_id=self.job_yaml["metadata"]["name"],
-            cmds=cmds,
-            arguments=arguments,
-            labels=all_labels,
-        )
+            # selecting appropriate pod values.
+            all_labels = {}
+            all_labels.update(self.labels)
+            all_labels.update(job_obj["spec"]["template"]["metadata"].get("labels", {}))
+            image = self.image or job_obj["spec"]["template"]["spec"]["containers"][0]["image"]
+            cmds = self.cmds or job_obj["spec"]["template"]["spec"]["containers"][0]["command"]
+            arguments = self.arguments or job_obj["spec"]["template"]["spec"]["containers"][0]["args"]
 
-        pod.service_account_name = self.service_account_name
-        pod.secrets = self.secrets
-        pod.envs = self.env_vars
-        pod.image_pull_policy = self.image_pull_policy
-        pod.image_pull_secrets = self.image_pull_secrets
-        pod.annotations = self.annotations
-        pod.resources = self.resources
-        pod.affinity = self.affinity
-        pod.node_selectors = self.node_selectors
-        pod.hostnetwork = self.hostnetwork
-        pod.tolerations = self.tolerations
-        pod.configmaps = self.configmaps
-        pod.security_context = self.security_context
-        pod.pod_runtime_info_envs = self.pod_runtime_info_envs
-        pod.dnspolicy = self.dnspolicy
+            pod = gen.make_pod(
+                namespace=job_obj["metadata"]["namespace"],
+                image=image,
+                pod_id=job_obj["metadata"]["name"],
+                cmds=cmds,
+                arguments=arguments,
+                labels=all_labels,
+            )
 
-        pod_yaml = pod_request_factory.SimplePodRequestFactory().create(pod)
+            pod.service_account_name = self.service_account_name
+            pod.secrets = self.secrets
+            pod.envs = self.env_vars
+            pod.image_pull_policy = self.image_pull_policy
+            pod.image_pull_secrets = self.image_pull_secrets
+            pod.annotations = self.annotations
+            pod.resources = self.resources
+            pod.affinity = self.affinity
+            pod.node_selectors = self.node_selectors
+            pod.hostnetwork = self.hostnetwork
+            pod.tolerations = self.tolerations
+            pod.configmaps = self.configmaps
+            pod.security_context = self.security_context
+            pod.pod_runtime_info_envs = self.pod_runtime_info_envs
+            pod.dnspolicy = self.dnspolicy
 
-        # setting the pod template.
-        self.job_yaml["spec"]["template"]["spec"] = pod_yaml["spec"]
+            # old pod generation.. moving to new one
+            pod_body = pod_request_factory.SimplePodRequestFactory().create(pod)
+        else:
+            pod_body = k8s.V1Pod(
+                api_version="v1",
+                kind="Pod",
+                metadata=k8s.V1ObjectMeta(
+                    namespace=self.namespace,
+                    labels=self.labels,
+                    name="legacy",
+                    annotations=self.annotations,
+                ),
+                spec=k8s.V1PodSpec(
+                    node_selector=self.node_selectors,
+                    affinity=self.affinity,
+                    tolerations=self.tolerations,
+                    init_containers=self.init_containers,
+                    containers=[
+                        k8s.V1Container(
+                            image=self.image,
+                            name="main",
+                            command=self.cmds,
+                            ports=self.ports,
+                            resources=self.resources,
+                            volume_mounts=self.volume_mounts,
+                            args=self.arguments,
+                            env=None
+                            if self.env_vars is None
+                            else [{"name": k, "value": self.env_vars[k]} for k in self.env_vars.keys()],
+                            env_from=self.env_from,
+                        )
+                    ],
+                    image_pull_secrets=self.image_pull_secrets,
+                    service_account_name=self.service_account_name,
+                    host_network=self.hostnetwork,
+                    security_context=self.security_context,
+                    dns_policy=self.dnspolicy,
+                    scheduler_name=self.schedulername,
+                    restart_policy="Never",
+                    priority_class_name=self.priority_class_name,
+                    volumes=self.volumes,
+                ),
+            )
+            pod_body = {
+                "metadata": self.job_runner.client.api_client.sanitize_for_serialization(pod_body.metadata),
+                "spec": self.job_runner.client.api_client.sanitize_for_serialization(pod_body.spec),
+            }
 
+        # reset the name
+        del self.body[0]["metadata"]["name"]
+        self.body[0]["spec"]["template"] = pod_body
+        self.job_runner.prepare_body(True)
